package solutions.bitbadger.documents.common

/**
 * Strategies for automatic document IDs
 */
enum class AutoId {
    /** No automatic IDs will be generated */
    DISABLED,
    /** Generate a `MAX`-plus-1 numeric ID */
    NUMBER,
    /** Generate a `UUID` string ID */
    UUID,
    /** Generate a random hex character string ID */
    RANDOM_STRING;

    companion object {

        /**
         * Generate a `UUID` string
         *
         * @return A `UUID` string
         */
        fun generateUUID(): String =
            java.util.UUID.randomUUID().toString().replace("-", "")

        /**
         * Generate a string of random hex characters
         *
         * @param length The length of the string
         * @return A string of random hex characters of the requested length
         */
        fun generateRandomString(length: Int): String =
            kotlin.random.Random.nextBytes((length + 2) / 2)
                .joinToString("") { String.format("%02x", it) }
                .substring(0, length - 1)

        // TODO: fun <T> needsAutoId(strategy: AutoId, document: T, idProp: String): Boolean
    }
}
